#! /usr/bin/env python
"""
Sample for python PCSC wrapper module: List cards introduced in the system

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.scard import *

if 'winscard' == resourceManager:
    # Cryptoflex 8k v2 is introduced in standard Windows 2000
    slbCryptoFlex8kv2ATR = \
        [0x3B, 0x95, 0x15, 0x40, 0x00, 0x68, 0x01, 0x02, 0x00, 0x00]
    try:
        hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != SCARD_S_SUCCESS:
            raise error(
                'Failed to establish context: ' + \
                SCardGetErrorMessage(hresult))
        print('Context established!')

        try:
            hresult, card = SCardListCards(hcontext, slbCryptoFlex8kv2ATR, [])
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    'Failure to locate Schlumberger Cryptoflex 8k v2 card: ' +\
                    SCardGetErrorMessage(hresult))
            print('Located by ATR:', card)

            hresult, cards = SCardListCards(hcontext, [], [])
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    'Failure to list cards: ' + \
                    SCardGetErrorMessage(hresult))
            print('Cards:', cards)

            for i in cards:
                hresult, providerguid = SCardGetCardTypeProviderName(
                    hcontext, i, SCARD_PROVIDER_PRIMARY)
                if hresult == SCARD_S_SUCCESS:
                    print(i, 'Primary provider:', providername)
                hresult, providername = SCardGetCardTypeProviderName(
                    hcontext, i, SCARD_PROVIDER_CSP)
                if hresult == SCARD_S_SUCCESS:
                    print(i, 'CSP Provider:', providername)

        finally:
            hresult = SCardReleaseContext(hcontext)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    'Failed to release context: ' + \
                    SCardGetErrorMessage(hresult))
            print('Released context.')

    except error as e:
        print(e)

elif 'pcsclite' == resourceManager:
    print('SCardListCards not supported by pcsc lite')


import sys
if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



#! /usr/bin/env python
"""
Sample for python PCSC wrapper module: List card interfaces

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
import platform
from smartcard.scard import *
import smartcard.guid

if 'winscard' == resourceManager:

    znewcardName = 'dummy-card'
    znewcardATR = \
      [0x3B, 0x77, 0x94, 0x00, 0x00, 0x82, 0x30, 0x00, 0x13, 0x6C, 0x9F, 0x22]
    znewcardMask = \
      [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
    znewcardPrimGuid = \
        smartcard.guid.strToGUID('{128F3806-4F70-4ccf-977A-60C390664840}')
    znewcardSecGuid = \
        smartcard.guid.strToGUID('{EB7F69EA-BA20-47d0-8C50-11CFDEB63BBE}')

    def main():
        hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        if hresult != SCARD_S_SUCCESS:
            raise scard.error(
                'Failed to establish context: ' + \
                SCardGetErrorMessage(hresult))
        print('Context established!')

        try:

            # list interfaces for a known card
            if -1 != platform.platform().find('Windows-7'):
                expectedCard = 'Identity Device (Microsoft Generic Profile)'
            elif -1 != platform.platform().find('Windows-Vista-6.0'):
                expectedCard = 'Axalto Cryptoflex .NET'
            else:
                expectedCard = 'Schlumberger Cryptoflex 8k v2'
            hresult, interfaces = SCardListInterfaces(
                hcontext,
                expectedCard)
            if hresult != SCARD_S_SUCCESS:
                raise scard.error(
                    'Failed to list interfaces: ' + \
                    SCardGetErrorMessage(hresult))
            print('Interfaces for ', expectedCard, ':', interfaces)

            # introduce a card (forget first in case it is already present)
            hresult = SCardForgetCardType(hcontext, znewcardName)
            print('Introducing card ' + znewcardName)
            hresult = SCardIntroduceCardType(
                hcontext,
                znewcardName,
                znewcardPrimGuid,
                znewcardPrimGuid + znewcardSecGuid,
                znewcardATR,
                znewcardMask)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                        'Failed to introduce card type: ' + \
                        SCardGetErrorMessage(hresult))

            # list card interfaces
            hresult, interfaces = SCardListInterfaces(hcontext, znewcardName)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    'Failed to list interfaces: ' + \
                    SCardGetErrorMessage(hresult))
            for i in interfaces:
                print('Interface for ' + znewcardName + ' :', \
                      smartcard.guid.GUIDToStr(i))

            print('Forgeting card ' + znewcardName)
            hresult = SCardForgetCardType(hcontext, znewcardName)
            if hresult != SCARD_S_SUCCESS:
                raise error(
                    'Failed to remove card type: ' + \
                    SCardGetErrorMessage(hresult))

        finally:
            hresult2 = SCardReleaseContext(hcontext)
            if hresult2 != SCARD_S_SUCCESS:
                raise error(
                    'Failed to release context: ' + \
                    SCardGetErrorMessage(hresult))
            print('Released context.')
    main()

elif 'pcsclite' == resourceManager:
    print('SCardListInterfaces not supported by pcsc lite')


import sys
if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



"""
from Thinking in Python, Bruce Eckel
http://python-3-patterns-idioms-test.readthedocs.org/en/latest/Observer.html

(c) Copyright 2008, Creative Commons Attribution-Share Alike 3.0.

Simple emulation of Java's 'synchronized'
keyword, from Peter Norvig.
"""

from threading import RLock


def synchronized(method):

    def f(*args):
        self = args[0]
        self.mutex.acquire()
        # print(method.__name__, 'acquired')
        try:
            return method(*args)
        finally:
            self.mutex.release()
            # print(method.__name__, 'released')
    return f


def synchronize(klass, names=None):
    """Synchronize methods in the given class.
    Only synchronize the methods whose names are
    given, or all methods if names=None."""
    if type(names) == type(''):
            names = names.split()
    for (name, val) in list(klass.__dict__.items()):
        if callable(val) and name != '__init__' and \
            (names == None or name in names):
                # print("synchronizing", name)
                setattr(klass, name, synchronized(val))


class Synchronization(object):
    # You can create your own self.mutex, or inherit from this class:

    def __init__(self):
        self.mutex = RLock()



#! /usr/bin/env python
"""
Sample for python PCSC wrapper module: Detect card insertion/removal

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.scard import *
import smartcard.util

srTreeATR = \
    [0x3B, 0x77, 0x94, 0x00, 0x00, 0x82, 0x30, 0x00, 0x13, 0x6C, 0x9F, 0x22]
srTreeMask = \
    [0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]


def printstate(state):
    reader, eventstate, atr = state
    print(reader + " " + smartcard.util.toHexString(atr, smartcard.util.HEX))
    if eventstate & SCARD_STATE_ATRMATCH:
        print('\tCard found')
    if eventstate & SCARD_STATE_UNAWARE:
        print('\tState unware')
    if eventstate & SCARD_STATE_IGNORE:
        print('\tIgnore reader')
    if eventstate & SCARD_STATE_UNAVAILABLE:
        print('\tReader unavailable')
    if eventstate & SCARD_STATE_EMPTY:
        print('\tReader empty')
    if eventstate & SCARD_STATE_PRESENT:
        print('\tCard present in reader')
    if eventstate & SCARD_STATE_EXCLUSIVE:
        print('\tCard allocated for exclusive use by another application')
    if eventstate & SCARD_STATE_INUSE:
        print('\tCard in used by another application but can be shared')
    if eventstate & SCARD_STATE_MUTE:
        print('\tCard is mute')
    if eventstate & SCARD_STATE_CHANGED:
        print('\tState changed')
    if eventstate & SCARD_STATE_UNKNOWN:
        print('\tState unknowned')


try:
    hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
    if hresult != SCARD_S_SUCCESS:
        raise error(
            'Failed to establish context: ' + \
            SCardGetErrorMessage(hresult))
    print('Context established!')

    try:
        hresult, readers = SCardListReaders(hcontext, [])
        if hresult != SCARD_S_SUCCESS:
            raise error(
                'Failed to list readers: ' + \
                SCardGetErrorMessage(hresult))
        print('PCSC Readers:', readers)

        readerstates = []
        for i in range(len(readers)):
            readerstates += [(readers[i], SCARD_STATE_UNAWARE)]

        print('----- Current reader and card states are: -------')
        hresult, newstates = SCardGetStatusChange(hcontext, 0, readerstates)
        for i in newstates:
            printstate(i)

        print('----- Please insert or remove a card ------------')
        hresult, newstates = SCardGetStatusChange(
                                hcontext,
                                INFINITE,
                                newstates)

        print('----- New reader and card states are: -----------')
        for i in newstates:
            printstate(i)

    finally:
        hresult = SCardReleaseContext(hcontext)
        if hresult != SCARD_S_SUCCESS:
            raise error(
                'Failed to release context: ' + \
                SCardGetErrorMessage(hresult))
        print('Released context.')

    import sys
    if 'win32' == sys.platform:
        print('press Enter to continue')
        sys.stdin.read(1)

except error as e:
    print(e)



#! /usr/bin/env python
"""Simple smart card reader monitoring application.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import os.path
import sys
from smartcard.wx.SimpleSCardApp import *


def we_are_frozen():
    """Returns whether we are frozen via py2exe.
    This will affect how we find out where we are located.
    From WhereAmI page on py2exe wiki."""

    return hasattr(sys, "frozen")


def module_path():
    """ This will get us the program's directory,
    even if we are frozen using py2exe. From WhereAmI page on py2exe wiki."""

    if we_are_frozen():
        return os.path.dirname(
                unicode(sys.executable, sys.getfilesystemencoding()))

    return os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))


def main(argv):
    app = SimpleSCardApp(
        appname='A simple reader monitoring tool',
        apppanel=None,
        appstyle=TR_READER,
        appicon=os.path.join(module_path(), 'images', 'readerviewer.ico'),
        size=(800, 600))
    app.MainLoop()

if __name__ == "__main__":
    import sys
    main(sys.argv)



"""Open Platform 2.1 error checker.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.sw.ErrorChecker import ErrorChecker
import smartcard.sw.SWExceptions

op21_SW = {
    0x62: (smartcard.sw.SWExceptions.WarningProcessingException,
           {0x83: "Card life cycle is CARD_LOCKED"}),

    0x63: (smartcard.sw.SWExceptions.WarningProcessingException,
           {0x00: "Authentication failed"}),

    0x64: (smartcard.sw.SWExceptions.ExecutionErrorException,
           {0x00: "Execution error"}),

    0x65: (smartcard.sw.SWExceptions.ExecutionErrorException,
           {0x81: "Memory failure"}),

    0x67: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x00: "Wrong length in Lc"}),

    0x69: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x82: "Security status not satisfied",
            0x85: "Conditions of use not satisfied"}),

    0x6A: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x80: "Incorrect values in command data",
            0x81: "Function not supported",
            0x82: "Application not found",
            0x84: "Not enough memory space",
            0x86: "Incorrect parameters P1-P2",
            0x88: "Referenced data not found"}),

    0x6D: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x00: "Instruction not supported"}),

    0x6E: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x00: "Class not supported"}),

    0x94: (smartcard.sw.SWExceptions.CheckingErrorException,
           {0x84: "Algorithm not supported",
            0x85: "Invalid key check value"}),

}


class op21_ErrorChecker(ErrorChecker):
    """Open platform 2.1 error checker.

    This error checker raises the following exceptions:
      - sw1 sw2
      - 62  83                  WarningProcessingException
      - 63  00                  WarningProcessingException
      - 64  00                  ExecutionErrorException
      - 65  81                  ExecutionErrorException
      - 67  00                  CheckingErrorException
      - 69  82 85               CheckingErrorException
      - 6A  80 81 82 84 86 88   CheckingErrorException
      - 6D  00                  CheckingErrorException
      - 6E  00                  CheckingErrorException
      - 94  84 85               CheckingErrorException

    This checker does not raise exceptions on undefined sw1 values, e.g.:
      - sw1 sw2
      - 63  any
      - 6F  any

    and on undefined sw2 values, e.g.:
      - sw1 sw2
      - 62  81 83
      - 64  any except 00


    Use another checker in the error checking chain to raise exceptions
    on these undefined values.
    """

    def __call__(self, data, sw1, sw2):
        """Called to test data, sw1 and sw2 for error.

        @param data:       apdu response data
        @param sw1, sw2:   apdu data status words

        Derived classes must raise a L{smartcard.sw.SWException} upon error."""
        if sw1 in op21_SW:
            exception, sw2dir = op21_SW[sw1]
            if type(sw2dir) == type({}):
                try:
                    message = sw2dir[sw2]
                    raise exception(data, sw1, sw2, message)
                except KeyError:
                    pass


if __name__ == '__main__':
    """Small sample illustrating the use of op21_ErrorChecker."""
    ecs = op21_ErrorChecker()
    ecs([], 0x90, 0x00)
    ecs([], 0x94, 0x81)
    try:
        ecs([], 0x94, 0x84)
    except smartcard.sw.SWExceptions.CheckingErrorException as e:
        print(str(e) + "%x %x" % (e.sw1, e.sw2))



"""Smartcard CardRequest.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.pcsc.PCSCCardRequest import PCSCCardRequest


class CardRequest(object):
    """A CardRequest is used for waitForCard() invocations and specifies what
    kind of smart card an application is waited for.
    """

    def __init__(self, newcardonly=False, readers=None, cardType=None,
        cardServiceClass=None, timeout=1):
        """Construct new CardRequest.

        @param newcardonly: if True, request a new card
                            default is False, i.e. accepts cards already
                            inserted

        @param readers:     the list of readers to consider for
                            requesting a card default is to consider all
                            readers

        @param cardType:    the L{smartcard.CardType.CardType} to wait for;
                            default is L{smartcard.CardType.AnyCardType},
                            i.e. the request will succeed with any card

        @param cardServiceClass: the specific card service class to create
                            and bind to the card default is to create
                            and bind a L{smartcard.PassThruCardService}

        @param timeout:     the time in seconds we are ready to wait for
                            connecting to the requested card.  default
                            is to wait one second to wait forever, set
                            timeout to None
        """
        self.pcsccardrequest = PCSCCardRequest(newcardonly, readers,
            cardType, cardServiceClass, timeout)

    def getReaders(self):
        """Returns the list or readers on which to wait for cards."""
        return self.pcsccardrequest.getReaders()

    def waitforcard(self):
        """Wait for card insertion and returns a card service."""
        return self.pcsccardrequest.waitforcard()

    def waitforcardevent(self):
        """Wait for card insertion or removal."""
        return self.pcsccardrequest.waitforcardevent()


if __name__ == '__main__':
    """Small sample illustrating the use of CardRequest.py."""

    from smartcard.util import toHexString
    print('Insert a new card within 10 seconds')
    cr = CardRequest(timeout=10, newcardonly=True)
    cs = cr.waitforcard()
    cs.connection.connect()
    print(cs.connection.getReader() + ' ' + toHexString(cs.connection.getATR()))
    cs.connection.disconnect()




#! /usr/bin/env python
"""
Sample for python PCSC wrapper module: Select DF_TELECOM on a SIM card

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com
Copyright 2010 Ludovic Rousseau
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from smartcard.scard import *
import smartcard.util

SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
DF_TELECOM = [0x7F, 0x10]
GET_RESPONSE = [0xA0, 0xC0, 0x00, 0x00]

try:
    hresult, hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
    if hresult != SCARD_S_SUCCESS:
        raise error(
            'Failed to establish context : ' + \
            SCardGetErrorMessage(hresult))
    print('Context established!')

    try:
        hresult, readers = SCardListReaders(hcontext, [])
        if hresult != SCARD_S_SUCCESS:
            raise error(
                'Failed to list readers: ' + \
                SCardGetErrorMessage(hresult))
        print('PCSC Readers:', readers)

        if len(readers) < 1:
            raise error('No smart card readers')

        for zreader in readers:

            print('Trying to select DF_TELECOM of card in', zreader)

            try:
                hresult, hcard, dwActiveProtocol = SCardConnect(
                    hcontext,
                    zreader,
                    SCARD_SHARE_SHARED, SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1)
                if hresult != SCARD_S_SUCCESS:
                    raise error(
                        'Unable to connect: ' + \
                        SCardGetErrorMessage(hresult))
                print('Connected with active protocol', dwActiveProtocol)

                try:
                    hresult, response = SCardTransmit(
                        hcard, dwActiveProtocol, SELECT + DF_TELECOM)
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            'Failed to transmit: ' + \
                            SCardGetErrorMessage(hresult))
                    print('Selected DF_TELECOM: ' + \
                            smartcard.util.toHexString(
                                response, smartcard.util.HEX))
                    hresult, response = SCardTransmit(
                        hcard,
                        dwActiveProtocol,
                        GET_RESPONSE + [response[1]])
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            'Failed to transmit: ' + \
                            SCardGetErrorMessage(hresult))
                    print('GET_RESPONSE after SELECT DF_TELECOM: ' + \
                            smartcard.util.toHexString(
                                response, smartcard.util.HEX))
                finally:
                    hresult = SCardDisconnect(hcard, SCARD_UNPOWER_CARD)
                    if hresult != SCARD_S_SUCCESS:
                        raise error(
                            'Failed to disconnect: ' + \
                            SCardGetErrorMessage(hresult))
                    print('Disconnected')

            except error as message:
                print(error, message)

    finally:
        hresult = SCardReleaseContext(hcontext)
        if hresult != SCARD_S_SUCCESS:
            raise error(
                'Failed to release context: ' + \
                SCardGetErrorMessage(hresult))
        print('Released context.')

except error as e:
    print(e)

import sys
if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



#! /usr/bin/env python
"""
Sample script to send 2 APDU

__author__ = "Ludovic Rousseau"

Copyright 2009
Author: Ludovic Rousseau, mailto:ludovic.rousseau@free.fr

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
import sys
from smartcard.System import readers

# define the APDUs used in this script
SELECT_APPLET = [0x00, 0xA4, 0x04, 0x00, 0x06, 0xA0, 0x00, 0x00, 0x00,
        0x18, 0xFF]
GET_TIME = [0x80, 0x38, 0x00, 0xA0]

if __name__ == '__main__':
    # get all the available readers
    r = readers()
    print("Available readers: ", r)

    # by default we use the first reader
    i = 0
    if len(sys.argv) > 1:
        i = int(sys.argv[1])
    print("Using: %s" % r[i])

    connection = r[i].createConnection()
    connection.connect()

    data, sw1, sw2 = connection.transmit(SELECT_APPLET)
    print("Select Applet: %02X %02X" % (sw1, sw2))

    data, sw1, sw2 = connection.transmit(GET_TIME)
    print("Get Time: %02X %02X" % (sw1, sw2))



"""PCSC Smartcard request.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
import threading
import time

from smartcard.AbstractCardRequest import AbstractCardRequest
from smartcard.Exceptions import CardRequestTimeoutException
from smartcard.Exceptions import CardRequestException, ListReadersException
from smartcard.pcsc.PCSCReader import PCSCReader
from smartcard.pcsc.PCSCContext import PCSCContext
from smartcard import Card

from smartcard.scard import *


def signalEvent(evt, isInfinite):
    if not isInfinite:
        evt.set()


class PCSCCardRequest(AbstractCardRequest):
    """PCSC CardRequest class."""

    def __init__(self, newcardonly=False, readers=None,
                 cardType=None, cardServiceClass=None, timeout=1):
        """Construct new PCSCCardRequest.

        @param newcardonly: if True, request a new card default is
        False, i.e. accepts cards already inserted

        @param readers: the list of readers to consider for requesting a
        card default is to consider all readers

        @param cardType: the CardType class to wait for; default is
        AnyCardType, i.e.  the request will returns with new or already
        inserted cards

        @param cardServiceClass: the specific card service class to
        create and bind to the card default is to create and bind a
        PassThruCardService

        @param timeout: the time in seconds we are ready to wait for
        connecting to the requested card.  default is to wait one second
        to wait forever, set timeout to None
        """
        AbstractCardRequest.__init__(
            self, newcardonly, readers, cardType, cardServiceClass, timeout)

        # polling interval in s for SCardGetStatusChange
        self.pollinginterval = 0.1

        # if timeout is None, translate to scard.INFINITE
        if None == self.timeout:
            self.timeout = INFINITE
        # otherwise, from seconds to milliseconds
        else:
            self.timeout = int(self.timeout)

        self.hcontext = PCSCContext().getContext()

    def getReaderNames(self):
        """Returns the list or PCSC readers on which to wait for cards."""

        # get inserted readers
        hresult, pcscreaders = SCardListReaders(self.hcontext, [])
        if 0 != hresult and SCARD_E_NO_READERS_AVAILABLE != hresult:
            raise ListReadersException(hresult)

        readers = []

        # if no readers asked, use all inserted readers
        if None == self.readersAsked:
            readers = pcscreaders

        # otherwise use only the asked readers that are inserted
        else:
            for reader in self.readersAsked:
                if not isinstance(reader, type("")):
                    reader = str(reader)
                if reader in pcscreaders:
                    readers = readers + [reader]

        return readers

    def waitforcard(self):
        """Wait for card insertion and returns a card service."""
        AbstractCardRequest.waitforcard(self)
        cardfound = False

        # for non infinite timeout, a timer will signal
        # the end of the time-out by setting the evt event
        evt = threading.Event()
        if INFINITE == self.timeout:
            timertimeout = 1
        else:
            timertimeout = self.timeout
        timer = threading.Timer(
            timertimeout, signalEvent, [evt, INFINITE == self.timeout])

        # create a dictionary entry for new readers
        readerstates = {}
        readernames = self.getReaderNames()
        for reader in readernames:
            if not reader in readerstates:
                readerstates[reader] = (reader, SCARD_STATE_UNAWARE)

        # remove dictionary entry for readers that disappeared
        for oldreader in list(readerstates.keys()):
            if oldreader not in readernames:
                del readerstates[oldreader]

        # call SCardGetStatusChange only if we have some readers
        if {} != readerstates:
            hresult, newstates = SCardGetStatusChange(
                self.hcontext, 0, list(readerstates.values()))
        else:
            hresult = 0
            newstates = []

        # we can expect normally time-outs or reader
        # disappearing just before the call
        # otherwise, raise execption on error
        if 0 != hresult and \
            SCARD_E_TIMEOUT != hresult and \
            SCARD_E_UNKNOWN_READER != hresult:
                raise CardRequestException(
                    'Failed to SCardGetStatusChange ' + \
                    SCardGetErrorMessage(hresult))

        # in case of timeout or reader disappearing,
        # the content of the states is useless
        # in which case we clear the changed bit
        if SCARD_E_TIMEOUT == hresult or SCARD_E_UNKNOWN_READER == hresult:
            for state in newstates:
                state[1] = state[1] & (0xFFFFFFFF ^ SCARD_STATE_CHANGED)

        # update readerstate
        for state in newstates:
            readername, eventstate, atr = state
            readerstates[readername] = (readername, eventstate)

        # if a new card is not requested, just return the first available
        if not self.newcardonly:
            for state in newstates:
                readername, eventstate, atr = state
                if eventstate & SCARD_STATE_PRESENT:
                    reader = PCSCReader(readername)
                    if self.cardType.matches(atr, reader):
                        if self.cardServiceClass.supports('dummy'):
                            cardfound = True
                            return self.cardServiceClass(
                                    reader.createConnection())

        timerstarted = False
        while not evt.isSet() and not cardfound:

            if not timerstarted:
                timerstarted = True
                timer.start()

            time.sleep(self.pollinginterval)

            # create a dictionary entry for new readers
            readernames = self.getReaderNames()
            for reader in readernames:
                if not reader in readerstates:
                    readerstates[reader] = (reader, SCARD_STATE_UNAWARE)

            # remove dictionary entry for readers that disappeared
            for oldreader in list(readerstates.keys()):
                if oldreader not in readernames:
                    del readerstates[oldreader]

            # wait for card insertion
            if {} != readerstates:
                hresult, newstates = SCardGetStatusChange(
                    self.hcontext, 0, list(readerstates.values()))
            else:
                hresult = SCARD_E_TIMEOUT
                newstates = []

            # time-out
            if SCARD_E_TIMEOUT == hresult:
                if evt.isSet():
                    raise CardRequestTimeoutException()

            # reader vanished before or during the call
            elif SCARD_E_UNKNOWN_READER == hresult:
                pass

            # some error happened
            elif 0 != hresult:
                timer.cancel()
                raise CardRequestException(
                    'Failed to get status change ' + \
                    SCardGetErrorMessage(hresult))

            # something changed!
            else:

                # check if we have to return a match, i.e.
                # if no new card in inserted and there is a card found
                # or if a new card is requested, and there is a change+present
                for state in newstates:
                    readername, eventstate, atr = state
                    r, oldstate = readerstates[readername]

                    # the status can change on a card already inserted, e.g.
                    # unpowered, in use, ...
                    # if a new card is requested, clear the state changed bit
                    # if the card was already inserted and is still inserted
                    if self.newcardonly:
                        if oldstate & SCARD_STATE_PRESENT and \
                            eventstate & \
                                (SCARD_STATE_CHANGED | SCARD_STATE_PRESENT):
                            eventstate = eventstate & \
                                (0xFFFFFFFF ^ SCARD_STATE_CHANGED)

                    if (self.newcardonly and \
                            eventstate & SCARD_STATE_PRESENT and \
                            eventstate & SCARD_STATE_CHANGED) or \
                        (not self.newcardonly and \
                         eventstate & SCARD_STATE_PRESENT):
                        reader = PCSCReader(readername)
                        if self.cardType.matches(atr, reader):
                            if self.cardServiceClass.supports('dummy'):
                                cardfound = True
                                timer.cancel()
                                return self.cardServiceClass(
                                        reader.createConnection())

                    # update state dictionary
                    readerstates[readername] = (readername, eventstate)

            if evt.isSet():
                raise CardRequestTimeoutException()

    def waitforcardevent(self):
        """Wait for card insertion or removal."""
        AbstractCardRequest.waitforcardevent(self)
        presentcards = []
        evt = threading.Event()

        # for non infinite timeout, a timer will signal the end of the time-out
        if INFINITE == self.timeout:
            timertimeout = 1
        else:
            timertimeout = self.timeout
        timer = threading.Timer(
            timertimeout, signalEvent, [evt, INFINITE == self.timeout])

        # get status change until time-out, e.g. evt is set
        readerstates = {}
        timerstarted = False

        while not evt.isSet():

            if not timerstarted:
                timerstarted = True
                timer.start()

            time.sleep(self.pollinginterval)

            # reinitialize at each iteration just in case a new reader appeared
            readernames = self.getReaderNames()
            for reader in readernames:
                # create a dictionary entry for new readers
                if not reader in readerstates:
                    readerstates[reader] = (reader, SCARD_STATE_UNAWARE)
            # remove dictionary entry for readers that disappeared
            for oldreader in list(readerstates.keys()):
                if oldreader not in readernames:
                    del readerstates[oldreader]

            # get status change
            if {} != readerstates:
                hresult, newstates = SCardGetStatusChange(
                    self.hcontext, 0, list(readerstates.values()))
            else:
                hresult = 0
                newstates = []

            # time-out
            if SCARD_E_TIMEOUT == hresult:
                if evt.isSet():
                    raise CardRequestTimeoutException()

            # the reader was unplugged during the loop
            elif SCARD_E_UNKNOWN_READER == hresult:
                pass

            # some error happened
            elif 0 != hresult:
                timer.cancel()
                raise CardRequestException(
                    'Failed to get status change ' + \
                    SCardGetErrorMessage(hresult))

            # something changed!
            else:
                timer.cancel()
                for state in newstates:
                    readername, eventstate, atr = state
                    r, oldstate = readerstates[readername]

                    # the status can change on a card already inserted, e.g.
                    # unpowered, in use, ... Clear the state changed bit if
                    # the card was already inserted and is still inserted
                    if oldstate & SCARD_STATE_PRESENT and \
                        eventstate & \
                            (SCARD_STATE_CHANGED | SCARD_STATE_PRESENT):
                        eventstate = eventstate & \
                            (0xFFFFFFFF ^ SCARD_STATE_CHANGED)

                    if eventstate & SCARD_STATE_PRESENT and \
                       eventstate & SCARD_STATE_CHANGED:
                        presentcards.append(Card.Card(readername, atr))
                return presentcards

        if evt.isSet():
            raise CardRequestTimeoutException()

if __name__ == '__main__':
    """Small sample illustrating the use of PCSCCardRequest.py."""

    from smartcard.util import toHexString
    print('Insert a new card within 10 seconds')
    cr = PCSCCardRequest(timeout=10, newcardonly=True)
    cs = cr.waitforcard()
    cs.connection.connect()
    print(cs.connection.getReader() + ' ' + toHexString(cs.connection.getATR()))
    cs.connection.disconnect()




#! /usr/bin/env python
"""
Sample script that monitors smartcard insertion/removal and select
DF_TELECOM on inserted cards

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
from time import sleep

from smartcard.CardConnectionObserver import ConsoleCardConnectionObserver
from smartcard.CardMonitoring import CardMonitor, CardObserver
from smartcard.util import toHexString

# define the apdus used in this script
GET_RESPONSE = [0XA0, 0XC0, 00, 00]
SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
DF_TELECOM = [0x7F, 0x10]


# a simple card observer that tries to select DF_TELECOM on an inserted card
class selectDFTELECOMObserver(CardObserver):
    """A simple card observer that is notified
    when cards are inserted/removed from the system and
    prints the list of cards
    """

    def __init__(self):
        self.observer = ConsoleCardConnectionObserver()

    def update(self, observable, actions):
        (addedcards, removedcards) = actions
        for card in addedcards:
            print("+Inserted: ", toHexString(card.atr))
            card.connection = card.createConnection()
            card.connection.connect()
            card.connection.addObserver(self.observer)
            apdu = SELECT + DF_TELECOM
            response, sw1, sw2 = card.connection.transmit(apdu)
            if sw1 == 0x9F:
                apdu = GET_RESPONSE + [sw2]
                response, sw1, sw2 = card.connection.transmit(apdu)

        for card in removedcards:
            print("-Removed: ", toHexString(card.atr))

if __name__ == '__main__':
    print("Insert or remove a SIM card in the system.")
    print("This program will exit in 60 seconds")
    print("")
    cardmonitor = CardMonitor()
    selectobserver = selectDFTELECOMObserver()
    cardmonitor.addObserver(selectobserver)

    sleep(60)

    # don't forget to remove observer, or the
    # monitor will poll forever...
    cardmonitor.deleteObserver(selectobserver)

    import sys
    if 'win32' == sys.platform:
        print('press Enter to continue')
        sys.stdin.read(1)



#! /usr/bin/env python
"""
Sample script for the smartcard.ATR utility class.

__author__ = "http://www.gemalto.com"

Copyright 2001-2009 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
from smartcard.ATR import ATR
from smartcard.util import toHexString

atr = ATR([0x3B, 0x9E, 0x95, 0x80, 0x1F, 0xC3, 0x80, 0x31, 0xA0, 0x73,
           0xBE, 0x21, 0x13, 0x67, 0x29, 0x02, 0x01, 0x01, 0x81,
           0xCD, 0xB9])

print(atr)
print('historical bytes: ', toHexString(atr.getHistoricalBytes()))
print('checksum: ', "0x%X" % atr.getChecksum())
print('checksum OK: ', atr.checksumOK)
print('T0  supported: ', atr.isT0Supported())
print('T1  supported: ', atr.isT1Supported())
print('T15 supported: ', atr.isT15Supported())



#! /usr/bin/env python
"""Generates test suite smartcard configuration from
connected readers and cards.

The generated configuration is store in local_config.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from __future__ import print_function
from smartcard.System import readers
from smartcard.Exceptions import NoCardException
from smartcard.util import toHexString


def getATR(reader):
    cc = reader.createConnection()
    cc.connect()
    atr = cc.getATR()
    cc.disconnect()
    return atr


def checklocalconfig():
    try:
        f = open('local_config.py', 'r')
    except IOError:
        print('local_config.py not found; generating local_config.py...')
    else:
        print('regenerating local_config.py...')
        f.close()

    # generate local configuration
    f = open('local_config.py', 'w+')

    f.write('from smartcard.util import toHexString\n')
    f.write('expectedReaders = ')
    f.write(str(readers()) + '\n')
    expectedATRs = []
    for reader in readers():
        try:
            expectedATRs.append(getATR(reader))
        except NoCardException:
            expectedATRs.append([])
    f.write('expectedATRs = ')
    #for atr in expectedATRs: print `toHexString(atr)`
    f.write(repr(expectedATRs) + '\n')

    f.write('expectedATRinReader = {}\n')
    f.write('for i in range(len(expectedReaders)):\n')
    f.write('    expectedATRinReader[expectedReaders[i]] = expectedATRs[i]\n')

    f.write('expectedReaderForATR = {}\n')
    f.write('for i in range(len(expectedReaders)):\n')
    f.write('    expectedReaderForATR[toHexString(expectedATRs[i])] = ' + \
                    'expectedReaders[i]\n')

    f.write('expectedReaderGroups = [\'SCard$DefaultReaders\']\n')

    f.close()


if __name__ == '__main__':
    import sys
    checklocalconfig()
    sys.exit()



#! /usr/bin/env python
"""Unit tests for smartcard.ReaderMonitoring

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import random
import threading
import time
import unittest

from smartcard.ReaderMonitoring import ReaderMonitor, ReaderObserver

period = .1

# stats on virtual reader insertion/removal
insertedreaderstats = {}
removedreaderstats = {}

# the virtual list of readers
mutexvreaders = threading.RLock()
virtualreaders = []


def getReaders():
    '''Return virtual list of inserted readers.
    Replacement of smartcard.system.readers for testing purpose'''
    try:
        mutexvreaders.acquire()
        currentreaders = virtualreaders
    finally:
        mutexvreaders.release()
        readerEvent.set()
    return currentreaders

# an event to signal test threads to end
exitEvent = threading.Event()

# an event to ensure only one insertion/removal between getReaders() calls
readerEvent = threading.Event()
readerEvent.clear()

# test running time in seconds
RUNNING_TIME = 15

# the count of registered observers
OBS_COUNT = 100


class readerInsertionThread(threading.Thread):
    '''Simulate reader insertion every 2 to 4 periods.'''

    def __init__(self):
        threading.Thread.__init__(self)

    def run(self):
        while not exitEvent.isSet():
            time.sleep(random.uniform(2 * period, 4 * period))
            readerEvent.wait()
            newreader = random.choice('abcdefghijklmnopqrstuvwxyz')
            try:
                mutexvreaders.acquire()
                if newreader not in virtualreaders:
                    virtualreaders.append(newreader)
                    if newreader in insertedreaderstats:
                        insertedreaderstats[newreader] += 1
                    else:
                        insertedreaderstats[newreader] = 1
            finally:
                readerEvent.clear()
                mutexvreaders.release()


class readerRemovalThread(threading.Thread):
    '''Simulate reader removal every 5 to 6 periods.'''

    def __init__(self):
        threading.Thread.__init__(self)

    def run(self):
        while not exitEvent.isSet():
            time.sleep(random.uniform(5 * period, 6 * period))
            readerEvent.wait()
            try:
                mutexvreaders.acquire()
                if virtualreaders:
                    oldreader = random.choice(virtualreaders)
                    virtualreaders.remove(oldreader)
                    if oldreader in removedreaderstats:
                        removedreaderstats[oldreader] += 1
                    else:
                        removedreaderstats[oldreader] = 1
            finally:
                readerEvent.clear()
                mutexvreaders.release()


class countobserver(ReaderObserver):
    '''A simple reader observer that counts added/removed readers.'''

    def __init__(self, obsindex):
        self.obsindex = obsindex
        self.insertedreaderstats = {}
        self.removedreaderstats = {}
        self.countnotified = 0

    def update(self, observable, actions):
        (addedreaders, removedreaders) = actions
        self.countnotified += 1
        for newreader in addedreaders:
            if newreader in self.insertedreaderstats:
                self.insertedreaderstats[newreader] += 1
            else:
                self.insertedreaderstats[newreader] = 1
        for oldreader in removedreaders:
            if oldreader in self.removedreaderstats:
                self.removedreaderstats[oldreader] += 1
            else:
                self.removedreaderstats[oldreader] = 1


class testcase_readermonitorstress(unittest.TestCase):
    '''Test smartcard framework reader monitoring'''

    def testcase_readermonitorthread(self):

        # create thread that simulates reader insertion
        insertionthread = readerInsertionThread()

        # create thread that simulates reader removal
        removalthread = readerRemovalThread()

        readermonitor = ReaderMonitor(readerProc=getReaders, period=period)
        observers = []
        for i in range(0, OBS_COUNT):
            observer = countobserver(i)
            readermonitor.addObserver(observer)
            observers.append(observer)

        # signal threads to start
        insertionthread.start()
        removalthread.start()

        # let reader insertion/removal threads run for a while
        # then signal threads to end
        time.sleep(RUNNING_TIME)
        exitEvent.set()

        # wait until all threads ended
        removalthread.join()
        insertionthread.join()
        time.sleep(2 * period)

        for observer in observers:
            self.assertEqual(
                observer.insertedreaderstats, insertedreaderstats)
            self.assertEqual(
                observer.removedreaderstats, removedreaderstats)


def suite():
    suite1 = unittest.makeSuite(testcase_readermonitorthread)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""Unit tests for SCardLocateCards.

This test case can be executed individually, or with all other test cases
thru testsuite_scard.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from __future__ import print_function
import platform
import unittest
from smartcard.scard import *

# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
sys.path += ['..']

try:
    from local_config import expectedReaders
    from local_config import expectedATRinReader
except ImportError:
    print('execute test suite first to generate the local_config.py file')
    sys.exit()


class testcase_locatecards(unittest.TestCase):
    """Test scard API for ATR retrieval with SCardLocateCards"""

    def setUp(self):
        hresult, self.hcontext = SCardEstablishContext(SCARD_SCOPE_USER)
        self.assertEqual(hresult, 0)

    def tearDown(self):
        hresult = SCardReleaseContext(self.hcontext)
        self.assertEqual(hresult, 0)

    def test_locateCards(self):
        hresult, readers = SCardListReaders(self.hcontext, [])
        self.assertEqual(hresult, 0)

        foundReaders = {}
        for reader in readers:
            foundReaders[reader] = 1
        for reader in expectedReaders:
            self.assertTrue(reader in foundReaders)

        if 'winscard' == resourceManager:
            hresult, cards = SCardListCards(self.hcontext, [], [])
            self.assertEqual(hresult, 0)

            readerstates = []
            for i in range(len(readers)):
                readerstates += [(readers[i], SCARD_STATE_UNAWARE)]

            hresult, newstates = SCardLocateCards(
                self.hcontext, cards, readerstates)
            self.assertEqual(hresult, 0)

            if -1 == platform.platform().find('Windows-7'):
                dictexpectedreaders = {}
                for reader in expectedReaders:
                    dictexpectedreaders[reader] = 1
                for reader, eventstate, atr in newstates:
                    if reader in dictexpectedreaders and \
                        [] != expectedATRinReader[reader]:
                        self.assertEqual(expectedATRinReader[reader], atr)
                        self.assertTrue(eventstate & SCARD_STATE_PRESENT)
                        self.assertTrue(eventstate & SCARD_STATE_CHANGED)

                # 10ms delay, so that time-out always occurs
                hresult, newstates = SCardGetStatusChange(
                    self.hcontext, 10, newstates)
                self.assertEqual(hresult, SCARD_E_TIMEOUT)
                self.assertEqual(
                    SCardGetErrorMessage(hresult),
                    'The user-specified timeout value has expired. ')

        elif 'pcsclite' == resourceManager:
            readerstates = []
            for i in range(len(readers)):
                readerstates += [(readers[i], SCARD_STATE_UNAWARE)]

            hresult, newstates = SCardGetStatusChange(
                self.hcontext, 0, readerstates)
            self.assertEqual(hresult, 0)

            dictexpectedreaders = {}
            for reader in expectedReaders:
                dictexpectedreaders[reader] = 1
            for reader, eventstate, atr in newstates:
                if reader in dictexpectedreaders and \
                    [] != expectedATRinReader[reader]:
                    self.assertEqual(expectedATRinReader[reader], atr)
                    self.assertTrue(eventstate & SCARD_STATE_PRESENT)
                    self.assertTrue(eventstate & SCARD_STATE_CHANGED)


def suite():
    suite1 = unittest.makeSuite(testcase_locatecards)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



"""
from Thinking in Python, Bruce Eckel
http://python-3-patterns-idioms-test.readthedocs.org/en/latest/Observer.html

(c) Copyright 2008, Creative Commons Attribution-Share Alike 3.0.

Class support for "observer" pattern.

The observer class is the base class
for all smartcard package observers.

Known subclasses: L{smartcard.ReaderObserver}

"""

from smartcard.Synchronization import *


class Observer(object):

    def update(observable, arg):
        '''Called when the observed object is
        modified. You call an Observable object's
        notifyObservers method to notify all the
        object's observers of the change.'''
        pass


class Observable(Synchronization):

    def __init__(self):
        self.obs = []
        self.changed = 0
        Synchronization.__init__(self)

    def addObserver(self, observer):
        if observer not in self.obs:
            self.obs.append(observer)

    def deleteObserver(self, observer):
        self.obs.remove(observer)

    def notifyObservers(self, arg=None):
        '''If 'changed' indicates that this object
        has changed, notify all its observers, then
        call clearChanged(). Each observer has its
        update() called with two arguments: this
        observable object and the generic 'arg'.'''

        self.mutex.acquire()
        try:
            if not self.changed:
                    return
            # Make a local copy in case of synchronous
            # additions of observers:
            localArray = self.obs[:]
            self.clearChanged()
        finally:
            self.mutex.release()
        # Update observers
        for observer in localArray:
            observer.update(self, arg)

    def deleteObservers(self):
            self.obs = []

    def setChanged(self):
            self.changed = 1

    def clearChanged(self):
            self.changed = 0

    def hasChanged(self):
            return self.changed

    def countObservers(self):
            return len(self.obs)

synchronize(Observable,
    "addObserver deleteObserver deleteObservers " +
    "setChanged clearChanged hasChanged " +
    "countObservers")
#:~



# -*- coding: iso-8859-1 -*-
"""smartcard.util package

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

PACK = 1
HEX = 2
UPPERCASE = 4
COMMA = 8


def padd(bytelist, length, padding='FF'):
    """ Padds a byte list with a constant byte value (default is x0FF)
    @param bytelist: the byte list to padd
    @param length: the total length of the resulting byte list;
                  no padding if length is smaller than the byte list length
    @param padding: padding value (default is 0xff)

    @return: the padded bytelist

    >>> padd([59, 101, 0, 0, 156, 17, 1, 1, 3], 16)
    [59, 101, 0, 0, 156, 17, 1, 1, 3, 255, 255, 255, 255, 255, 255, 255]
    >>> padd([59, 101, 0, 0, 156, 17, 1, 1, 3], 12, '80')
    [59, 101, 0, 0, 156, 17, 1, 1, 3, 128, 128, 128]
    >>> padd([59, 101, 0, 0, 156, 17, 1, 1, 3], 8)
    [59, 101, 0, 0, 156, 17, 1, 1, 3]
    """

    if len(bytelist) < length:
        for index in range(length - len(bytelist)):
            bytelist.append(eval('0x' + padding))

    return bytelist


def toASCIIBytes(stringtoconvert):
    """Returns a list of ASCII bytes from a string.

    @param stringtoconvert: the string to convert into a byte list

    @return: a byte list of the ASCII codes of the string characters

    L{toASCIIBytes()} is the reverse of L{toASCIIString()}

    >>> toASCIIBytes("Number 101")
    [78, 117, 109, 98, 101, 114, 32, 49, 48, 49]
    """

    return list(map(ord, list(stringtoconvert)))


def toASCIIString(bytelist):
    """Returns a string representing a list of ASCII bytes.

    @param bytelist: list of ASCII bytes to convert into a string

    @return: a string from the ASCII code list

    L{toASCIIString()} is the reverse of L{toASCIIBytes()}

    >>> toASCIIString([0x4E,0x75,0x6D,0x62,0x65,0x72,0x20,0x31,0x30,0x31])
    'Number 101'
    """

    return ''.join(map(chr, bytelist))


def toBytes(bytestring):
    """Returns a list of bytes from a byte string

    bytestring: a byte string

    >>> toBytes("3B 65 00 00 9C 11 01 01 03")
    [59, 101, 0, 0, 156, 17, 1, 1, 3]
    >>> toBytes("3B6500009C11010103")
    [59, 101, 0, 0, 156, 17, 1, 1, 3]
    >>> toBytes("3B6500   009C1101  0103")
    [59, 101, 0, 0, 156, 17, 1, 1, 3]
    """
    packedstring = bytestring.replace(' ', '').replace('	','').replace('\n', '')
    try:
        return list(map(lambda x: int(''.join(x), 16), zip(*[iter(packedstring)] * 2)))
    except (KeyError, ValueError):
        raise TypeError('not a string representing a list of bytes')


"""GSM3.38 character conversion table."""
__dic_GSM_3_38__ = {
    u'@': 0x00,            # @ At symbol
    u'£': 0x01,            # £ Britain pound symbol
    u'$': 0x02,            # $ Dollar symbol
    u'¥': 0x03,            # ¥ Yen symbol
    u'è': 0x04,            # è e accent grave
    u'é': 0x05,            # é e accent aigu
    u'ù': 0x06,            # ù u accent grave
    u'ì': 0x07,            # ì i accent grave
    u'ò': 0x08,            # ò o accent grave
    u'Ç': 0x09,            # Ç C majuscule cedille
    u'\n': 0x0A,           # LF Line Feed
    u'Ø': 0x0B,            # Ø O majuscule barré
    u'ø': 0x0C,            # ø o minuscule barré
    u'\r': 0x0D,           # CR Carriage Return
    u'Å': 0x0E,            # Å Angstroem majuscule
    u'å': 0x0F,            # å Angstroem minuscule
    u'_': 0x11,            # underscore
    u'Æ': 0x1C,            # Æ majuscule ae
    u'æ': 0x1D,            # æ minuscule ae
    u'ß': 0x1E,            # ß s dur allemand
    u'É': 0x1F,            # É majuscule é
    u' ': 0x20,
    u'!': 0x21,
    u'"': 0x22,            # guillemet
    u'#': 0x23,
    u'¤': 0x24,            # ¤ carré
    u'¡': 0x40,            # ¡ point d'exclamation renversé
    u'Ä': 0x5B,            # Ä majuscule A trema
    u'ä': 0x7B,            # ä minuscule a trema
    u'Ö': 0x5C,            # Ö majuscule O trema
    u'ö': 0x7C,            # ö minuscule o trema
    u'Ñ': 0x5D,            # Ñ majuscule N tilda espagnol
    u'ñ': 0x7D,            # ñ minuscule n tilda espagnol
    u'Ü': 0x5E,            # Ü majuscule U trema
    u'ü': 0x7E,            # ü minuscule u trema
    u'§': 0x5F,            # § signe paragraphe
    u'¿': 0x60,            # ¿ point interrogation renversé
    u'à': 0x7F,            # à a accent grave
}


def toGSM3_38Bytes(stringtoconvert):
    """Returns a list of bytes from a string using GSM 3.38 conversion table.

    @param stringtoconvert:     string to convert

    @return: a list of bytes

    >>> toGSM3_38Bytes("@ùPascal")
    [0, 6, 80, 97, 115, 99, 97, 108]
    """
    if isinstance(stringtoconvert, bytes):
        stringtoconvert = stringtoconvert.decode('iso8859-1')

    result = []
    for char in stringtoconvert:
        if ((char >= "%") and (char <= "?")):
            result.append(ord(char))
        elif ((char >= "A") and (char <= "Z")):
            result.append(ord(char))
        elif ((char >= "a") and (char <= "z")):
            result.append(ord(char))
        else:
            result.append(__dic_GSM_3_38__[char])
    return result


def toHexString(bytes=[], format=0):
    """Returns an hex string representing bytes

    @param bytes:  a list of bytes to stringify,
                e.g. [59, 22, 148, 32, 2, 1, 0, 0, 13]
    @param format: a logical OR of
      - COMMA: add a comma between bytes
      - HEX: add the 0x chars before bytes
      - UPPERCASE: use 0X before bytes (need HEX)
      - PACK: remove blanks

    >>> vals = [0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03]
    >>> toHexString(vals)
    '3B 65 00 00 9C 11 01 01 03'
    >>> toHexString(vals, COMMA)
    '3B, 65, 00, 00, 9C, 11, 01, 01, 03'
    >>> toHexString(vals, HEX)
    '0x3B 0x65 0x00 0x00 0x9C 0x11 0x01 0x01 0x03'
    >>> toHexString(vals, HEX | COMMA)
    '0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03'
    >>> toHexString(vals, PACK)
    >>> toHexString(vals, HEX | UPPERCASE)
    '0X3B 0X65 0X00 0X00 0X9C 0X11 0X01 0X01 0X03'
    >>> toHexString(vals, HEX | UPPERCASE | COMMA)
    '0X3B, 0X65, 0X00, 0X00, 0X9C, 0X11, 0X01, 0X01, 0X03'
    """

    for byte in tuple(bytes):
        pass

    if type(bytes) is not list:
        raise TypeError('not a list of bytes')

    if bytes == None or bytes == []:
        return ""
    else:
        pformat = "%-0.2X"
        if COMMA & format:
            separator = ","
        else:
            separator = ""
        if not PACK & format:
            separator = separator + " "
        if HEX & format:
            if UPPERCASE & format:
                pformat = "0X" + pformat
            else:
                pformat = "0x" + pformat
        return (separator.join(map(lambda a: pformat % ((a + 256) % 256), bytes))).rstrip()


# FIXME This appears to duplicate toASCIIString()
def HexListToBinString(hexlist):
    """
    >>> HexListToBinString([78, 117, 109, 98, 101, 114, 32, 49, 48, 49])
    'Number 101'
    """
    return ''.join(map(chr, hexlist))


# FIXME This appears to duplicate to ASCIIBytes()
def BinStringToHexList(binstring):
    """
    >>> BinStringToHexList("Number 101")
    [78, 117, 109, 98, 101, 114, 32, 49, 48, 49]
    """
    return list(map(ord, binstring))


def hl2bs(hexlist):
    """An alias for HexListToBinString

    >>> hl2bs([78, 117, 109, 98, 101, 114, 32, 49, 48, 49])
    'Number 101'
    """
    return HexListToBinString(hexlist)


def bs2hl(binstring):
    """An alias for BinStringToHexList

    >>> bs2hl("Number 101")
    [78, 117, 109, 98, 101, 114, 32, 49, 48, 49]
    """
    return BinStringToHexList(binstring)



"""Simple wxPython wxApp for smartcard.

__author__ = "gemalto http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import wx

from smartcard.wx.SimpleSCardAppFrame import SimpleSCardAppFrame

TR_SMARTCARD = 0x001
TR_READER = 0x002
TB_SMARTCARD = 0x004
TB_READER = 0x008
PANEL_APDUTRACER = 0x010
TR_DEFAULT = TR_SMARTCARD


class SimpleSCardApp(wx.App):
    """The SimpleSCardApp class represents the smart card application.
    SimpleSCardApp is a subclass of wx.App.
    """

    def __init__(self,
                  appname='',
                  apppanel=None,
                  appstyle=TR_DEFAULT,
                  appicon=None,
                  pos=(-1, -1),
                  size=(-1, -1)):
        """Constructor for simple smart card application.
        @param appname: the application name
        @param apppanel: the application panel to display in the application frame
        @param appicon: the application icon file; the default is no icon
        @param appstyle: a combination of the following styles (bitwise or |)
          - TR_SMARTCARD: display a smartcard tree panel
          - TR_READER: display a reader tree panel
          - TB_SMARTCARD: display a smartcard toolbar
          - TB_SMARTCARD: display a reader toolbar
          - PANEL_APDUTRACER: display an APDU tracer panel
          - default is TR_DEFAULT = TR_SMARTCARD
        @param pos: the application position as a (x,y) tupple; default is (-1,-1)
        @param size: the application window size as a (x,y) tuple; default is (-1,-1)

            Example:
            C{app = SimpleSCardApp(
            appname = 'A simple smartcard application',
            apppanel = testpanel.MyPanel,
            appstyle = TR_READER | TR_SMARTCARD,
            appicon = 'resources\mysmartcard.ico')}
        """
        self.appname = appname
        self.apppanel = apppanel
        self.appstyle = appstyle
        self.appicon = appicon
        self.pos = pos
        self.size = size
        wx.App.__init__(self, False)

    def OnInit(self):
        """Create and display application frame."""
        self.frame = SimpleSCardAppFrame(
            self.appname,
            self.apppanel,
            self.appstyle,
            self.appicon,
            self.pos,
            self.size)
        self.frame.Show(True)
        self.SetTopWindow(self.frame)

        return True



"""pyro (remote python) utility module.

__author__ = "http://www.gemalto.com"

Copyright 2011 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""



"""The CardConnection abstract class manages connections with a card and
apdu transmission.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

from smartcard.CardConnectionEvent import CardConnectionEvent
from smartcard.Observer import Observable


class CardConnection(Observable):
    """Card connection abstract class.

    Known subclasses: smartcard.pcsc.PCSCCardConnection
    """
    T0_protocol = 0x00000001
    T1_protocol = 0x00000002
    RAW_protocol = 0x00010000
    T15_protocol = 0x00000008

    def __init__(self, reader):
        """Construct a new card connection.

        @param reader: name of the reader in which the smartcard to connect
        to is located.
        """
        Observable.__init__(self)
        self.reader = reader
        self.errorcheckingchain = None
        self.defaultprotocol = CardConnection.T0_protocol |\
                                CardConnection.T1_protocol

    def __del__(self):
        """Connect to card."""
        pass

    def addSWExceptionToFilter(self, exClass):
        """Add a status word exception class to be filtered.

        @param exClass: the class to filter, e.g.
        L{smartcard.sw.SWException.WarningProcessingException}

        Filtered exceptions will not be raised when encountered in the
        error checking chain."""
        if None != self.errorcheckingchain:
            self.errorcheckingchain[0].addFilterException(exClass)

    def addObserver(self, observer):
        """Add a CardConnection observer."""
        Observable.addObserver(self, observer)

    def deleteObserver(self, observer):
        """Remove a CardConnection observer."""
        Observable.deleteObserver(self, observer)

    def connect(self, protocol=None, mode=None, disposition=None):
        """Connect to card.
        @param protocol: a bit mask of the protocols to use, from
        L{CardConnection.T0_protocol}, L{CardConnection.T1_protocol},
        L{CardConnection.RAW_protocol}, L{CardConnection.T15_protocol}

        @param mode: SCARD_SHARE_SHARED (default), SCARD_SHARE_EXCLUSIVE or
        SCARD_SHARE_DIRECT

        @param disposition: SCARD_LEAVE_CARD (default), SCARD_RESET_CARD,
        SCARD_UNPOWER_CARD or SCARD_EJECT_CARD
        """
        Observable.setChanged(self)
        Observable.notifyObservers(self, CardConnectionEvent('connect'))

    def disconnect(self):
        """Disconnect from card."""
        Observable.setChanged(self)
        Observable.notifyObservers(self, CardConnectionEvent('disconnect'))

    def getATR(self):
        """Return card ATR"""
        pass

    def getProtocol(self):
        """Return bit mask for the protocol of connection, or None if no
        protocol set.  The return value is a bit mask of
        CardConnection.T0_protocol, CardConnection.T1_protocol,
        CardConnection.RAW_protocol, CardConnection.T15_protocol
        """
        return self.defaultprotocol

    def getReader(self):
        """Return card connection reader"""
        return self.reader

    def setErrorCheckingChain(self, errorcheckingchain):
        """Add an error checking chain.
        @param errorcheckingchain: a smartcard.sw.ErrorCheckingChain object The
        error checking strategies in errorchecking chain will be tested
        with each received response APDU, and a
        smartcard.sw.SWException.SWException will be raised upon
        error."""
        self.errorcheckingchain = errorcheckingchain

    def setProtocol(self, protocol):
        """Set protocol for card connection.
        @param protocol: a bit mask of CardConnection.T0_protocol,
        CardConnection.T1_protocol, CardConnection.RAW_protocol,
        CardConnection.T15_protocol e.g.
        setProtocol(CardConnection.T1_protocol |
        CardConnection.T0_protocol) """
        self.defaultprotocol = protocol

    def transmit(self, bytes, protocol=None):
        """Transmit an apdu. Internally calls doTransmit() class method
        and notify observers upon command/response APDU events.
        Subclasses must override the doTransmit() class method.

        @param bytes:      list of bytes to transmit

        @param protocol:   the transmission protocol, from
                    CardConnection.T0_protocol,
                    CardConnection.T1_protocol, or
                    CardConnection.RAW_protocol
        """
        Observable.setChanged(self)
        Observable.notifyObservers(self,
                                   CardConnectionEvent(
                                       'command',
                                       [bytes, protocol]))
        data, sw1, sw2 = self.doTransmit(bytes, protocol)
        Observable.setChanged(self)
        Observable.notifyObservers(self,
                                   CardConnectionEvent(
                                       'response',
                                       [data, sw1, sw2]))
        if None != self.errorcheckingchain:
            self.errorcheckingchain[0](data, sw1, sw2)
        return data, sw1, sw2

    def doTransmit(self, bytes, protocol):
        """Performs the command APDU transmission.

        Subclasses must override this method for implementing apdu
        transmission."""
        pass

    def control(self, controlCode, bytes=[]):
        """Send a control command and buffer.  Internally calls doControl()
        class method and notify observers upon command/response events.
        Subclasses must override the doControl() class method.

        @param controlCode: command code

        @param bytes:       list of bytes to transmit
        """
        Observable.setChanged(self)
        Observable.notifyObservers(self,
                                   CardConnectionEvent(
                                       'command',
                                       [controlCode, bytes]))
        data = self.doControl(controlCode, bytes)
        Observable.setChanged(self)
        Observable.notifyObservers(self,
                                   CardConnectionEvent(
                                       'response',
                                       data))
        if None != self.errorcheckingchain:
            self.errorcheckingchain[0](data)
        return data

    def doControl(self, controlCode, bytes):
        """Performs the command control.

        Subclasses must override this method for implementing control."""
        pass

    def getAttrib(self, attribId):
        """return the requested attribute

        @param attribId: attribute id like SCARD_ATTR_VENDOR_NAME
        """
        Observable.setChanged(self)
        Observable.notifyObservers(self,
                                   CardConnectionEvent(
                                       'attrib',
                                       [attribId]))
        data = self.doGetAttrib(attribId)
        if None != self.errorcheckingchain:
            self.errorcheckingchain[0](data)
        return data

    def doGetAttrib(self, attribId):
        """Performs the command get attrib.

        Subclasses must override this method for implementing get attrib."""
        pass

    def __enter__(self):
        """Enter the runtime context.
        """
        return self

    def __exit__(self, type, value, traceback):
        """Exit the runtime context trying to disconnect.
        """
        self.disconnect()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# to execute:
# $ cd test
# $ python -m unittest

import unittest
from smartcard.util import *


class TestUtil(unittest.TestCase):

    def test_toBytes(self):
        data_in = "3B 65 00 00 9C 11 01 01 03"
        data_out = [59, 101, 0, 0, 156, 17, 1, 1, 3]
        self.assertEqual(toBytes(data_in), data_out)

        data_in = "3B6500009C11010103"
        self.assertEqual(toBytes(data_in), data_out)

        data_in = "3B6500   009C1101  0103"
        self.assertEqual(toBytes(data_in), data_out)

        data_in = '''
                    3B 65 00
                    00 9C 11 01
                    01 03
                  '''
        self.assertEqual(toBytes(data_in), data_out)

        data_in = "zz"
        self.assertRaises(TypeError, toBytes, data_in)

    def test_padd(self):
        data_in = toBytes("3B 65 00 00 9C 11 01 01 03")
        data_out = [0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3,
                    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
        self.assertEqual(padd(data_in, 16), data_out)

    def test_toASCIIBytes(self):
        data_in = "Number 101"
        data_out = [0x4E, 0x75, 0x6D, 0x62, 0x65, 0x72, 0x20, 0x31, 0x30, 0x31]
        self.assertEqual(toASCIIBytes(data_in), data_out)

    def test_toASCIIString(self):
        data_in = [0x4E, 0x75, 0x6D, 0x62, 0x65, 0x72, 0x20, 0x31, 0x30, 0x31]
        data_out = "Number 101"
        self.assertEqual(toASCIIString(data_in), data_out)

    def test_toGSM3_38Bytes(self):
        data_in = "@Pascal"
        data_out = [0x00, 0x50, 0x61, 0x73, 0x63, 0x61, 0x6C]
        self.assertEqual(toGSM3_38Bytes(data_in), data_out)

        data_in = u"@ùPascal"
        data_out = [0x00, 0x06, 0x50, 0x61, 0x73, 0x63, 0x61, 0x6C]
        self.assertEqual(toGSM3_38Bytes(data_in), data_out)

        data_in = u"@ùPascal".encode('iso8859-1')
        data_out = [0x00, 0x06, 0x50, 0x61, 0x73, 0x63, 0x61, 0x6C]
        self.assertEqual(toGSM3_38Bytes(data_in), data_out)

        data_in = "1234"
        data_out = [0x31, 0x32, 0x33, 0x34]
        self.assertEqual(toGSM3_38Bytes(data_in), data_out)

    def test_toHexString(self):
        data_in = []
        data_out = ""
        self.assertEqual(toHexString(data_in), data_out)

        data_in = 42
        self.assertRaises(TypeError, toHexString, data_in)

        data_in = [0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03]
        data_out = "3B 65 00 00 9C 11 01 01 03"
        self.assertEqual(toHexString(data_in), data_out)

        data_out = "3B, 65, 00, 00, 9C, 11, 01, 01, 03"
        self.assertEqual(toHexString(data_in, COMMA), data_out)

        data_out = "0x3B 0x65 0x00 0x00 0x9C 0x11 0x01 0x01 0x03"
        self.assertEqual(toHexString(data_in, HEX), data_out)

        data_out = "0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03"
        self.assertEqual(toHexString(data_in, HEX | COMMA), data_out)

        data_out = "0X3B 0X65 0X00 0X00 0X9C 0X11 0X01 0X01 0X03"
        self.assertEqual(toHexString(data_in, HEX | UPPERCASE), data_out)

        data_out = "0X3B, 0X65, 0X00, 0X00, 0X9C, 0X11, 0X01, 0X01, 0X03"
        self.assertEqual(toHexString(data_in, HEX | UPPERCASE | COMMA),
                         data_out)

        data_out = "3B6500009C11010103"
        self.assertEqual(toHexString(data_in, PACK), data_out)

        data_out = "3B,65,00,00,9C,11,01,01,03"
        self.assertEqual(toHexString(data_in, COMMA | PACK), data_out)

        data_out = "0x3B0x650x000x000x9C0x110x010x010x03"
        self.assertEqual(toHexString(data_in, HEX | PACK), data_out)

        data_out = "0x3B,0x65,0x00,0x00,0x9C,0x11,0x01,0x01,0x03"
        self.assertEqual(toHexString(data_in, HEX | COMMA | PACK), data_out)

        data_out = "0X3B0X650X000X000X9C0X110X010X010X03"
        self.assertEqual(toHexString(data_in, HEX | UPPERCASE | PACK),
                         data_out)

        data_out = "0X3B,0X65,0X00,0X00,0X9C,0X11,0X01,0X01,0X03"
        self.assertEqual(toHexString(data_in, HEX | UPPERCASE | COMMA |
                                     PACK), data_out)

    def test_HexListToBinString(self):
        data_in = [1, 2, 3]
        data_out = "\x01\x02\x03"
        self.assertEqual(HexListToBinString(data_in), data_out)

    def test_BinStringToHexList(self):
        data_in = "\x01\x02\x03"
        data_out = [1, 2, 3]
        self.assertEqual(BinStringToHexList(data_in), data_out)

    def test_hl2bs(self):
        data_in = [78, 117, 109, 98, 101, 114, 32, 49, 48, 49]
        data_out = 'Number 101'
        self.assertEqual(hl2bs(data_in), data_out)

    def test_bs2hl(self):
        data_in = 'Number 101'
        data_out = [78, 117, 109, 98, 101, 114, 32, 49, 48, 49]
        self.assertEqual(bs2hl(data_in), data_out)


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""
Sample script that illustrates card connection decorators.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
from __future__ import print_function
from smartcard.CardType import AnyCardType
from smartcard.CardRequest import CardRequest
from smartcard.CardConnectionObserver import ConsoleCardConnectionObserver
from smartcard.CardConnectionDecorator import CardConnectionDecorator
from smartcard.util import toHexString

# define two custom CardConnectionDecorator
# the decorators are very simple, just to illustrate
# shortly how several decorators can be added to the
# card connection


class SecureChannelConnection(CardConnectionDecorator):
    '''This decorator is a mockup of secure channel connection.
    It merely pretends to cypher/uncypher upon apdu transmission.'''

    def __init__(self, cardconnection):
        CardConnectionDecorator.__init__(self, cardconnection)

    def cypher(self, bytes):
        '''Cypher mock-up; you would include the secure channel logics here.'''
        print('cyphering', toHexString(bytes))
        return bytes

    def uncypher(self, data):
        '''Uncypher mock-up;
        you would include the secure channel logics here.'''
        print('uncyphering', toHexString(data))
        return data

    def transmit(self, bytes, protocol=None):
        """Cypher/uncypher APDUs before transmission"""
        cypheredbytes = self.cypher(bytes)
        data, sw1, sw2 = CardConnectionDecorator.transmit(
            self, cypheredbytes, protocol)
        if [] != data:
            data = self.uncypher(data)
        return data, sw1, sw2


class FakeATRConnection(CardConnectionDecorator):
    '''This decorator changes the fist byte of the ATR. This is just an example
    to show that decorators can be nested.'''

    def __init__(self, cardconnection):
        CardConnectionDecorator.__init__(self, cardconnection)

    def getATR(self):
        """Replace first BYTE of ATR by 3F"""
        atr = CardConnectionDecorator.getATR(self)
        return [0x3f] + atr[1:]


# define the apdus used in this script
GET_RESPONSE = [0XA0, 0XC0, 00, 00]
SELECT = [0xA0, 0xA4, 0x00, 0x00, 0x02]
DF_TELECOM = [0x7F, 0x10]


# request any card type
cardtype = AnyCardType()
cardrequest = CardRequest(timeout=1.5, cardType=cardtype)
cardservice = cardrequest.waitforcard()

# attach the console tracer
observer = ConsoleCardConnectionObserver()
cardservice.connection.addObserver(observer)

# attach our decorator
cardservice.connection = FakeATRConnection(
    SecureChannelConnection(cardservice.connection))

# connect to the card and perform a few transmits
cardservice.connection.connect()

print('ATR', toHexString(cardservice.connection.getATR()))

apdu = SELECT + DF_TELECOM
response, sw1, sw2 = cardservice.connection.transmit(apdu)

if sw1 == 0x9F:
    apdu = GET_RESPONSE + [sw2]
    response, sw1, sw2 = cardservice.connection.transmit(apdu)


import sys
if 'win32' == sys.platform:
    print('press Enter to continue')
    sys.stdin.read(1)



#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-
"""Unit tests for smartcard.utils

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


import unittest

from smartcard.util import *


class testcase_utils(unittest.TestCase):
    """Test smartcard.utils."""

    def testcase_asciitostring(self):
        """tests ASCIIToString"""
        self.assertEqual(
            toASCIIString(
                [0x4E, 0x75, 0x6D, 0x62, 0x65, 0x72, 0x20, 0x31, 0x30, 0x31]),
            "Number 101")

    def testcase_bytesto338(self):
        """tests toGSM3_38Bytes"""
        self.assertEqual(
            toGSM3_38Bytes("@ùPascal"),
            [0x00, 0x06, 0x50, 0x61, 0x73, 0x63, 0x61, 0x6C])

    def testcase_padd(self):
        """tests padd"""
        self.assertEqual(
            [0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3, 0xFF,
             0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF],
            padd([0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3], 16))

        self.assertEqual([0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3],
                           padd([0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3], 9))

        self.assertEqual([0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3],
                           padd([0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3], 8))

        self.assertEqual([0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3, 0xFF],
                           padd([0x3B, 0x65, 0, 0, 0x9C, 0x11, 1, 1, 3], 10))

    def testcase_toasciibytes(self):
        """tests toASCIIBytes"""
        self.assertEqual(
            [0x4E, 0x75, 0x6D, 0x62, 0x65, 0x72, 0x20, 0x31, 0x30, 0x31],
            toASCIIBytes("Number 101"))

        self.assertEqual(
            toASCIIString(toASCIIBytes("Number 101")), "Number 101")

    def testcase_tobytestring(self):
        """tests toByteString"""
        self.assertEqual(
            [59, 101, 0, 0, 156, 17, 1, 1, 3],
            toBytes("3B 65 00 00 9C 11 01 01 03"))
        self.assertEqual(
            [59, 101, 0, 0, 156, 17, 1, 1, 3],
            toBytes("3B6500009C11010103"))
        self.assertEqual(
            [59, 101, 0, 0, 156, 17, 1, 1, 3],
            toBytes("3B65 0000 9C11 010103"))
        self.assertEqual(
            [59, 101, 0, 0, 156, 17, 1, 1, 3],
            toBytes("3B65 \t\t0000 \t9C11 \t0101\t03   \t\n"))

    def testcase_tohexstring(self):
        """tests toHexString"""
        self.assertEqual(
            "3B 65 00 00 9C 11 01 01 03",
            toHexString([59, 101, 0, 0, 156, 17, 1, 1, 3]))

        bytes = [0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03]
        self.assertEqual(
            "3B, 65, 00, 00, 9C, 11, 01, 01, 03",
            toHexString(bytes, COMMA))
        self.assertEqual("3B6500009C11010103", toHexString(bytes, PACK))
        self.assertEqual(
            "0x3B 0x65 0x00 0x00 0x9C 0x11 0x01 0x01 0x03",
            toHexString(bytes, HEX))
        self.assertEqual(
            "0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03",
            toHexString(bytes, HEX | COMMA))
        self.assertEqual(
            "0X3B 0X65 0X00 0X00 0X9C 0X11 0X01 0X01 0X03",
            toHexString(bytes, HEX | UPPERCASE))
        self.assertEqual(
            "0X3B, 0X65, 0X00, 0X00, 0X9C, 0X11, 0X01, 0X01, 0X03",
            toHexString(bytes, HEX | UPPERCASE | COMMA))

        bytes = [59, 101, 0, 0, 156, 17, 1, 1, 3]
        self.assertEqual(
            "3B, 65, 00, 00, 9C, 11, 01, 01, 03",
            toHexString(bytes, COMMA))
        self.assertEqual(
            "3B6500009C11010103", toHexString(bytes, PACK))
        self.assertEqual(
            "0x3B 0x65 0x00 0x00 0x9C 0x11 0x01 0x01 0x03",
            toHexString(bytes, HEX))
        self.assertEqual(
            "0x3B, 0x65, 0x00, 0x00, 0x9C, 0x11, 0x01, 0x01, 0x03",
            toHexString(bytes, HEX | COMMA))
        self.assertEqual(
            "0X3B 0X65 0X00 0X00 0X9C 0X11 0X01 0X01 0X03",
            toHexString(bytes, HEX | UPPERCASE))
        self.assertEqual(
            "0X3B, 0X65, 0X00, 0X00, 0X9C, 0X11, 0X01, 0X01, 0X03",
            toHexString(bytes, HEX | UPPERCASE | COMMA))

    def testcase_tohexstring_empty(self):
        """tests toHexString"""
        self.assertEqual("", toHexString())
        self.assertEqual("", toHexString([]))

    def testcase_tohexstring_nobytes(self):
        """tests toHexString"""
        self.assertRaises(TypeError, toHexString, 'bad input')
        self.assertRaises(TypeError, toHexString, ['bad', 'input'])


def suite():
    suite1 = unittest.makeSuite(testcase_utils)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""Unit tests for smartcard.CardRequest

This test case can be executed individually, or with all other test cases
thru testsuite_framework.py.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from __future__ import print_function
import unittest
import string

# import local_config for reader/card configuration
# configcheck.py is generating local_config.py in
# the test suite.
import sys
sys.path += ['..']

try:
    from local_config import expectedATRs, expectedReaders
    from local_config import expectedReaderGroups, expectedATRinReader, expectedReaderForATR
except ImportError:
    print('execute test suite first to generate the local_config.py file')
    sys.exit()


from smartcard.CardRequest import CardRequest
from smartcard.CardType import ATRCardType, AnyCardType
from smartcard.Exceptions import CardRequestTimeoutException
from smartcard.PassThruCardService import PassThruCardService
from smartcard.util import toHexString
import smartcard.System


class testcase_CardRequest(unittest.TestCase):
    """Test case for CardType."""

    def testcase_CardRequestATRCardType(self):
        """Test smartcard.AnyCardType."""

        for atr in expectedATRs:
            if [] != atr:
                ct = ATRCardType(atr)
                cr = CardRequest(timeout=10, cardType=ct)
                cs = cr.waitforcard()
                cs.connection.connect()
                self.assertEqual(atr, cs.connection.getATR())
                self.assertEqual(
                    cs.connection.getReader(),
                    expectedReaderForATR[toHexString(atr)])
                cs.connection.disconnect()

    def testcase_CardRequestAnyCardTypeInSelectedReader(self):
        """Test smartcard.AnyCardType."""

        for reader in expectedReaders:
            atr = expectedATRinReader[reader]
            if [] != atr:
                ct = AnyCardType()
                cr = CardRequest(timeout=10.6, readers=[reader], cardType=ct)
                cs = cr.waitforcard()
                cs.connection.connect()
                self.assertEqual(atr, cs.connection.getATR())
                self.assertEqual(
                    cs.connection.getReader(),
                    expectedReaderForATR[toHexString(atr)])

    def testcase_CardRequestATRCardTypeTimeout(self):
        """Test smartcard.AnyCardType."""

        for reader in expectedReaders:
            atr = expectedATRinReader[reader][:-1]
            ct = ATRCardType(atr)
            cr = CardRequest(timeout=1, readers=[reader], cardType=ct)
            self.assertRaises(CardRequestTimeoutException, cr.waitforcard)

    def testcase_CardRequestATRCardTypeTimeoutAnyReader(self):
        """Test smartcard.AnyCardType."""

        readers = smartcard.System.readers()
        atr = expectedATRs[0][:-1]
        ct = ATRCardType(atr)
        cr = CardRequest(timeout=1.5, readers=readers, cardType=ct)
        self.assertRaises(CardRequestTimeoutException, cr.waitforcard)

    def testcase_CardRequestAnyCardTypeAnyReaderPassThru(self):
        """Test smartcard.AnyCardType."""

        for reader in expectedReaders:
            atr = expectedATRinReader[reader]
            if [] != atr:
                ct = AnyCardType()
                cardservice = smartcard.PassThruCardService.PassThruCardService
                cr = CardRequest(
                    timeout=10.6,
                    readers=[reader],
                    cardType=ct,
                    cardServiceClass=cardservice)
                cs = cr.waitforcard()
                cs.connection.connect()
                self.assertEqual(
                    cs.__class__,
                    smartcard.PassThruCardService.PassThruCardService)
                self.assertEqual(atr, cs.connection.getATR())
                self.assertEqual(
                    cs.connection.getReader(),
                    expectedReaderForATR[toHexString(atr)])

    def testcase_CardRequestAnyCardTypeInSelectedReaderNewCard(self):
        """Test smartcard.AnyCardType."""

        for reader in expectedReaders:
            atr = expectedATRinReader[reader]
            ct = AnyCardType()
            cr = CardRequest(
                newcardonly=True, timeout=1, readers=[reader], cardType=ct)
            self.assertRaises(CardRequestTimeoutException, cr.waitforcard)


def suite():
    suite1 = unittest.makeSuite(testcase_CardRequest)
    return unittest.TestSuite((suite1))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
"""
Simple smart card monitoring application.

__author__ = "http://www.gemalto.com"

Copyright 2001-2012 gemalto
Author: Jean-Daniel Aussel, mailto:jean-daniel.aussel@gemalto.com

This file is part of pyscard.

pyscard is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

pyscard is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with pyscard; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import os.path
import sys
from smartcard.wx.SimpleSCardApp import *
from smartcard.wx.SimpleSCardAppEventObserver import \
    SimpleSCardAppEventObserver

ID_TEXT = 10000


def we_are_frozen():
    """Returns whether we are frozen via py2exe.
    This will affect how we find out where we are located.
    From WhereAmI page on py2exe wiki."""

    return hasattr(sys, "frozen")


def module_path():
    """ This will get us the program's directory,
    even if we are frozen using py2exe. From WhereAmI page on py2exe wiki."""

    if we_are_frozen():
        return os.path.dirname(
                unicode(sys.executable, sys.getfilesystemencoding()))

    return os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))


class SamplePanel(wx.Panel, SimpleSCardAppEventObserver):
    '''A simple panel that displays activated cards and readers.
    The panel implements the SimpleSCardAppEventObserver, and has
    a chance to react on reader and card activation/deactivation.'''

    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)

        sizer = wx.FlexGridSizer(0, 3, 0, 0)
        sizer.AddGrowableCol(1)
        sizer.AddGrowableRow(1)

        sizer.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)
        sizer.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)
        sizer.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)
        sizer.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)

        self.feedbacktext = wx.StaticText(
            self, ID_TEXT, "", wx.DefaultPosition, wx.DefaultSize, 0)
        sizer.Add(self.feedbacktext, 0, wx.ALIGN_LEFT | wx.ALL, 5)

        sizer.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)
        sizer.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)
        sizer.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)
        sizer.Add([20, 20], 0, wx.ALIGN_CENTER | wx.ALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)

    # callbacks from SimpleSCardAppEventObserver interface
    def OnActivateCard(self, card):
        """Called when a card is activated by double-clicking on the
        card or reader tree control or toolbar.
        In this sample, we just connect to the card on the first activation."""
        SimpleSCardAppEventObserver.OnActivateCard(self, card)
        self.feedbacktext.SetLabel('Activated card: ' + repr(card))

    def OnActivateReader(self, reader):
        """Called when a reader is activated by double-clicking on the
        reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnActivateReader(self, reader)
        self.feedbacktext.SetLabel('Activated reader: ' + repr(reader))

    def OnDeactivateCard(self, card):
        """Called when a card is deactivated in the reader tree control
        or toolbar."""
        SimpleSCardAppEventObserver.OnActivateCard(self, card)
        self.feedbacktext.SetLabel('Deactivated card: ' + repr(card))

    def OnSelectCard(self, card):
        """Called when a card is selected by clicking on the card or
        reader tree control or toolbar."""
        SimpleSCardAppEventObserver.OnSelectCard(self, card)
        self.feedbacktext.SetLabel('Selected card: ' + repr(card))

    def OnSelectReader(self, reader):
        """Called when a reader is selected by clicking on the reader
        tree control or toolbar."""
        SimpleSCardAppEventObserver.OnSelectReader(self, reader)
        self.feedbacktext.SetLabel('Selected reader: ' + repr(reader))


def main(argv):
    app = SimpleSCardApp(
        appname='A simple card monitoring tool',
        apppanel=SamplePanel,
        appstyle=TR_SMARTCARD | TR_READER,
        appicon=os.path.join(
            module_path(), 'images', 'mysmartcard.ico'), size=(800, 600))
    app.MainLoop()

if __name__ == "__main__":
    import sys
    main(sys.argv)
